@@ -31,14 +31,3 @@
 #' @usage data(stopwords_list)
 #' @format A named list, with names matching the languages used by SnowballC
 "stopwords_list"
-
-#' Dictionary with common ASCII emoticons
-#'
-#' Obtained from the Wikipedia List_of_emoticons page.
-#'
-#' @docType data
-#' @usage data(emoticon_dict)
-#' @format A data.frame with a "string" and "code" column.
-"emoticon_dict"
-
-
